//////////////////////////////////////////////////////////////////////
// IniFile.cpp: Implementation of the CIniFile class.				//
// Written by:	Adam Clauss		pandcc3@comwerx.net					//
// Rewritten by Matthias Weber	wmat@mbv.tu-chemnitz.de  02/2003	//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// IniFile.cpp: CIniFile-Klasse										//
//																	//
// Bedienungsanleitung:			erstellt von Weber Matthias 03/2003	//
//																	//
// ----------------------------------------------------------------	//
// INI-Struktur:	[keyname]										//
//					valuename=value	;comment						//
//					valuename=value	;comment						//
//																	//
//					[keyname]										//
//					valuename=value	;comment						//
//																	//
// Es werden 2 Listen angelegt										//
//	1x Namen der Keys (keynames)= names	( array of CString	)		//
//	1x Inhalt der Keys			= keys	( array of struct	)		//
//																	//
// ----------------------------------------------------------------	//
// Erstellung:														//
//																	//
//		Klasse.SetPath("datei.ini");								//
//		Klasse.Reset();					// Neuerstellen	der Daten	//
//		Klasse.SetValue("KEY_A","Wert1",50,true);					//
//		Klasse.SetValue("KEY_A","Wert2","hallo",true);				//
//		Klasse.SetValue("KEY_A","Wert3",100.4,true);				//
//		Klasse.SetValueComment("KEY_A","Wert2","Kommentar");		//
//		Klasse.SetValue("KEY_B","Wert1",50,true);					//
//		Klasse.SetValue("KEY_B","Wert2","hallo",true);				//
//		Klasse.SetValue("KEY_B","Wert3",100.4,true);				//
//		Klasse.SetValueComment("KEY_A","Wert2","Kommentar");		//
//		Klasse.SetValue("KEY_C","Wert1",50,true);					//
//		Klasse.SetValue("KEY_C","Wert2","hallo",true);				//
//		Klasse.SetValue("KEY_C","Wert3",100.4,true);				//
//		Klasse.SetValueComment("KEY_A","Wert2","Kommentar");		//
//			//bis jetzt wurde noch nix gespeichert					//
//			//man kann aber dennoch schon auf die Zellen zugreifen	//
//																	//
//		Klasse.WriteFile();				// Erstellung der Datei		//
//																	//
// ----------------------------------------------------------------	//
// Resultat: "datei.ini"											//
//																	//
// [KEY_A]															//
// Wert1=50			;Kommentar zum Wert								//
// Wert2=hallo														//
// Wert3=100.4														//
//																	//
// [KEY_B]															//
// Wert1=50			;Kommentar zum Wert								//
// Wert2=hallo														//
// Wert3=100.4														//
//																	//
// [KEY_C]															//
// Wert1=50			;Kommentar zum Wert								//
// Wert2=hallo														//
// Wert3=100.4														//
//																	//
// ----------------------------------------------------------------	//
// Lesen															//
//																	//
//		Klasse.SetPath("datei.ini");								//
//		Klasse.ReadFile();	//Prfen auf RckgabeCode==0			//
//							//nun kann auf Zellen zugegriffn werden	//
//																	//
// ----------------------------------------------------------------	//
// Zugriff															//
//		SetValue("KEY_A","Wert1",51);								// ndern der Zeile (Comment bleibt erhalten)
//		SetValue("KEY_A","Wert4",100);								// hinzufgen der Zeile (ohne Comment)
//		int ix;														//
//		ix=GetValueInt("KEY_A","Wert4");							// liefert wert4 aus [KEY_A] als Integer
//		double dx;													//
//		dx=GetValueDouble("KEY_A","Wert3");							// liefert wert3 aus [KEY_A] als Double
//		CString sx;													//
//		sx=GetValue("KEY_A","Wert2");								// liefert wert2 aus [KEY_A] als CString
//		SetValueComment("KEY_A","Wert4","Komentar");				// fgt zu wert4 aus [KEY_A] einen Kommentar
//		sx=GetValueComment("KEY_A","Wert4");						// liefert den Kommentar zu wert4 aus [KEY_A]
//		SetValueComment("KEY_A","Wert1","");						// lscht den Kommentar zu wert1 aus [KEY_A]
//																	//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"						
//#include "fstream.h"
#include <fstream> //added by S.Wei 10.08.2007
#include "IniFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CIniFile* CIniFile::m_pCIniFile;				//statischer pointer auf instanz 

CIniFile::CIniFile()							//default constructor
{	m_pCIniFile=this;							//"save in static pointer"
	TRACE("CIniFile::CIniFile()\n");		
}

CIniFile::CIniFile(CString inipath)				//constructor with filename
{	m_pCIniFile=this;							//"save in static pointer"
	TRACE("CIniFile::CIniFile(\""+inipath+"\")\n");	
	path = inipath;
}

CIniFile::~CIniFile()							//default destructor
{	
	TRACE("CIniFile::~CIniFile()\n");
	WriteFile();
}

void CIniFile::SetPath(CString newpath)
{	
	path = newpath;
}

CString CIniFile::GetPath() const
{
	return path;
}

bool CIniFile::ReadFile()
{
	CFile file;
	CFileStatus status;
	if (!file.GetStatus(path,status))
	{	AfxMessageBox(	"CIniFile::ReadFile()\n\nFehler bei .GetStatus() von "+path+".\n"+
						"Datei ist nicht vorhanden oder fehlerhaft.\n"+
						"Es werden keine Daten eingelesen.",MB_OK+MB_ICONERROR,0);
		return 0;
	}
	ifstream inifile;
	CString readinfo;
	inifile.open(path);
	int curkey = -1, curval = -1;
	if (inifile.fail())
	{	AfxMessageBox(	"CIniFile::ReadFile()\n\nFehler bei .open() von "+path+".\n"+
						"Datei ist nicht vorhanden oder fehlerhaft.\n"+
						"Es werden keine Daten eingelesen.",MB_OK+MB_ICONERROR,0);
		return 0;
	}
	CString keyname, valuename, value, comment;
	CString temp;
	
	while (getline(inifile,readinfo))
	{
		comment="";
		int index = readinfo.Find(';');			//comment found?
		if (index != -1) 
		{ 
			comment = readinfo.Right(readinfo.GetLength()-index-1); 
			readinfo = readinfo.Left(index);
		};
		readinfo.TrimRight();
		if (readinfo != "")
		{	if (readinfo[0] == '[' && readinfo[readinfo.GetLength()-1] == ']') 
			{									//if a section heading
				keyname = readinfo;
				keyname.TrimLeft('[');
				keyname.TrimRight(']');
			}
			else 
			{									//if a value
				valuename = readinfo.Left(readinfo.Find("="));
				value = readinfo; 
				value.Delete(0, valuename.GetLength() + 1 ); 
				valuename.TrimRight(); 
				value.TrimLeft(); 
				SetValue(keyname,valuename,value);
				SetValueComment(keyname,valuename,comment);
			}
		}
	}
	inifile.close();
	TRACE("CIniFile::ReadFile(): INI-Daten erfolgreich eingelesen...\n");
	return 1;
}

void CIniFile::WriteFile()
{
	ofstream inifile;
	inifile.open(path);
	for (int keynum = 0; keynum <= names.GetUpperBound(); keynum++)
	{
		if (keys[keynum].names.GetSize() != 0)
		{
			inifile << '[' << names[keynum] << ']' << endl;
			for (int valuenum = 0; valuenum <= keys[keynum].names.GetUpperBound(); valuenum++)
			{
				inifile << keys[keynum].names[valuenum] << "=" << keys[keynum].values[valuenum];
				if (keys[keynum].comment[valuenum]!="")	
				  inifile << "	;" << keys[keynum].comment[valuenum];
								
				if (valuenum != keys[keynum].names.GetUpperBound())
					inifile << endl;
				else
					if (keynum < names.GetSize())
						inifile << endl;
			}
			if (keynum < names.GetSize())
				inifile << endl;
		}
	}
	inifile.close();
}

// --------------------------------------------------------------------------------------------	
//
// Public Functions:	DatenZugriffe
//
// --------------------------------------------------------------------------------------------	

void CIniFile::Reset()
{
	keys.SetSize(0);
	names.SetSize(0);
}

int CIniFile::GetNumKeys() const						//returns number of keys currently in the ini
{
	return keys.GetSize();
}

int CIniFile::GetNumValues(CString keyname) const		//returns number of values stored for specified key, or -1 if key found
{
	int keynum = FindKey(keyname);
	if (keynum == -1)
		return -1;
	else
		return keys[keynum].names.GetSize();
}

CString CIniFile::GetValue(CString keyname, CString valuename) const			//get [keyname] valuename = ____
{	int keynum = FindKey(keyname), valuenum = FindValue(keynum,valuename);
	if (keynum == -1)
	{	TRACE("CIniFile::GetValue(): Unable to locate specified key "+keyname+"\n");
		return "";
	}
	if (valuenum == -1)
	{	TRACE("CIniFile::GetValue(): Unable to locate specified value "+valuename+"\n");
		return "";
	}
	return keys[keynum].values[valuenum];
}

int CIniFile::GetValueInt(CString keyname, CString valuename) const			//[keyname] valuename = 
{	
	return atoi(GetValue(keyname,valuename));
}

float CIniFile::GetValueDouble(CString keyname, CString valuename) const		//[keyname] valuename = 
{	
	return (float)atof(GetValue(keyname, valuename));
}

// Goddelnee! Geht das nicht mit sscanf()?
void CIniFile::GetValueTrFloat(CString keyname, CString valuename, float V[3]) const {	
	CString strvalue=GetValue(keyname, valuename);
	CString temp;
	if (strvalue.GetLength()<=1)
	{	V[0]=V[1]=V[2]=0.0f;
		return;
	};
	int index;
	strvalue.TrimLeft();	strvalue.TrimLeft('(');		strvalue.TrimLeft();
	
	index=strvalue.Find(',');
	if (index<0) {V[0]=V[1]=V[2]=0.0f;return;}
	temp=strvalue.Left(index);
	temp.TrimRight();
	V[0]=(float)atof(temp);
	strvalue=strvalue.Right(strvalue.GetLength()-index-1);
	
	index=strvalue.Find(',');
	if (index<0) {V[0]=V[1]=V[2]=0.0;return;}
	temp=strvalue.Left(index);
	temp.TrimRight();
	V[1]=(float)atof(temp);
	strvalue=strvalue.Right(strvalue.GetLength()-index-1);
	
	strvalue.TrimRight();	strvalue.TrimRight(')');	strvalue.TrimRight();
	V[2]=(float)atof(strvalue);
}


CString CIniFile::GetValueComment(CString keyname, CString valuename) const	//[keyname] valuename = value  ;___________
{	
	int keynum = FindKey(keyname), valuenum = FindValue(keynum,valuename);
	if (keynum == -1)
	{	TRACE("CIniFile::GetComment(): Unable to locate specified key "+keyname+"\n");
		return "";
	}
	if (valuenum == -1)
	{	TRACE("CIniFile::GetComment(): Unable to locate specified value "+valuename+"\n");
		return "";
	}
	CString temp;
	temp=keys[keynum].comment[valuenum];
	return keys[keynum].comment[valuenum];
}

bool CIniFile::SetValue(CString keyname, CString valuename, CString value, bool create)
{	
	//[keyname] valuename = value
	int keynum = FindKey(keyname), valuenum = 0;
	//find key
	if (keynum == -1) //if key doesn't exist
	{	//TRACE("CIniFile::SetValue(): Unable to locate specified key "+keyname);
		if (!create) //and user does not want to create it,
		{	//TRACE("\n");
			return 0; //stop entering this key
		};
		names.SetSize(names.GetSize()+1);
		keys.SetSize(keys.GetSize()+1);
		keynum = names.GetSize()-1;
		names[keynum] = keyname;
		//TRACE("... now created.\n");
	}

	//find value
	valuenum = FindValue(keynum,valuename);
	if (valuenum == -1) //if value doesn't exist
	{	//TRACE("CIniFile::SetValue(): Unable to locate specified value "+valuename);
		if (!create) // and user does not want to create it,
		{	//TRACE("\n");
			return 0; //stop entering this value
		};
		keys[keynum].names.SetSize(keys[keynum].names.GetSize()+1);
		keys[keynum].values.SetSize(keys[keynum].names.GetSize()+1);
		keys[keynum].comment.SetSize(keys[keynum].names.GetSize()+1);
		valuenum = keys[keynum].names.GetSize()-1;
		keys[keynum].names[valuenum] = valuename;
		keys[keynum].comment[valuenum] = "";
		//TRACE("... now created.\n");
	}
	keys[keynum].values[valuenum] = value;
	
	return 1;
}

bool CIniFile::SetValue(CString keyname, CString valuename, int value, bool create)
{	
	//[keyname] valuename = (int)value
	CString temp;
	temp.Format("%d",value);
	return SetValue(keyname, valuename, temp, create);
}

bool CIniFile::SetValue(CString keyname, CString valuename, float value, bool create) {	
	//[keyname] valuename = (double)value
 CString temp;
 temp.Format("%.6g",value);//%.(anz nachkommastellen)
 return SetValue(keyname, valuename, temp, create);
}

bool CIniFile::SetValue(CString keyname, CString valuename, const float V[3], bool create) {
 CString temp;
 temp.Format("%.4g,%.4g,%.4g",V[0],V[1],V[2]);		//%.(anz nachkommastellen)(float)
 return SetValue(keyname, valuename, temp, create);
}

bool CIniFile::SetValueComment(CString keyname, CString valuename, CString Comment)
{	
	//[keyname] valuename = value ;___________
	int keynum = FindKey(keyname), valuenum = 0;

	if (keynum == -1)
	{	TRACE("CIniFile::SetValueComment(): Unable to locate specified key "+keyname+"\n");
		return 0;			//if key doesn't exist
	}
	valuenum = FindValue(keynum,valuename);
	if (valuenum == -1)
	{	TRACE("CIniFile::SetValueComment(): Unable to locate specified value "+valuename+"\n");
		return 0;			//if value doesn't exist
	}
	keys[keynum].comment[valuenum] = Comment;
	return 1;
}

bool CIniFile::DeleteValue(CString keyname, CString valuename)
{
	int keynum = FindKey(keyname), valuenum = FindValue(keynum,valuename);
	if (keynum == -1 || valuenum == -1)
	{	TRACE("CIniFile::DeleteValue(): Unable to locate specified key "+keyname+" and value "+valuename+"\n");
		return 0;									//value/key does not exist
	}
	keys[keynum].names.RemoveAt(valuenum);
	keys[keynum].values.RemoveAt(valuenum);
	keys[keynum].comment.RemoveAt(valuenum);
	return 1;										//value successfully deleted
}

bool CIniFile::DeleteKey(CString keyname)
{
	int keynum = FindKey(keyname);
	if (keynum == -1)
	{	TRACE("CIniFile::DeleteKey(): Unable to locate specified key "+keyname+"\n");
		return 0;									//key does not exits
	};
	keys.RemoveAt(keynum);
	names.RemoveAt(keynum);
	return 1;										//key successfully deleted
}

int CIniFile::FindKey(CString keyname) const					//Eror = -1
{
	int keynum = 0;
	while ( keynum < keys.GetSize() && names[keynum] != keyname)
		keynum++;
	if (keynum == keys.GetSize())
		return -1;
	return keynum;
}


int CIniFile::FindValue(int keynum, CString valuename) const	//Eror = -1
{
	if (keynum == -1)
		return -1;
	int valuenum = 0;
	while (valuenum < keys[keynum].names.GetSize() && keys[keynum].names[valuenum] != valuename)
		valuenum++;
	if (valuenum == keys[keynum].names.GetSize())
		return -1;
	return valuenum;
}

bool CIniFile::getline(istream &is, CString &str) const
{	char buf[2048];
    is.getline(buf,2048);
    str = buf;
	if (is) return 1;
	else return 0;
}

bool CIniFile::IsKey(CString strKeyName) const
{	return (FindKey(strKeyName)>=0) ;
}

bool CIniFile::IsValue(CString strKeyName, CString strValueName) const
{	int nKey=FindKey(strKeyName);
	return (FindValue(nKey,strValueName)>=0);
}
