//////////////////////////////////////////////////////////////////////
// OpenGLFunctions.cpp: interface for COpenGLFuctions				//
// Written by Matthias Weber	wmat@mbv.tu-chemnitz.de  02/2003	//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// COpenGLFunctions.cpp: COpenGLFunctions-Klasse					//
//																	//
// Bedienungsanleitung:			erstellt von Weber Matthias 03/2003	//
//																	//
// ----------------------------------------------------------------	//
// ntige LIB's -> in Projekteinstellungen eintragen:				//
//		opengl32.lib glu32.lib glaux.lib							//
//																	//
// ----------------------------------------------------------------	//
// Instanz der Klasse erzeugen: im HeaderFile der Basisklasse 		//
//		COpenGLFunctions m_OpenGL									//
//																	//
// ----------------------------------------------------------------	//
// Initialisierung:  in OnCreate() der Basisklasse (CWnd)			//
//		m_OpenGL.m_hWnd=m_hWnd;										//
//		m_OpenGL.m_pDC = new CClientDC(this);						//
//		m_OpenGL.Init();											//
//																	//
// ----------------------------------------------------------------	//
// Aufrufe: in OnPaint() der Basisklasse (CWnd)						//
//		m_OpenGL.DrawScene();										//
//																	//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "math.h"
#include "IniFile.h"
#include "HexMotion.h"
#include <fstream> //changed by S.Wei 10.08.2007
#include "gl\gl.h"			//OpenGL - Standartbefehle	z.B. glVertex3d(), ...
#include "gl\glu.h"			//OpenGL - Utilities		z.B. gluPerspective(), ...
//#include "gl\glaux.h"		//OpenGL - Hilfsbibliothek	z.B. auxSolidSphere(), ...
#include "OpenGLFunctions.h"
#include "MainFrm.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

COpenGLFunctions*	COpenGLFunctions::m_pCOpenGLFunctions;				//statischer pointer auf instanz 


COpenGLFunctions::COpenGLFunctions()
{	
	TRACE("COpenGLFunctions::COpenGLFunctions()\n");
	m_pCOpenGLFunctions=NULL;//wird in Init() belegt
	m_pDC = NULL;
	m_hWnd= NULL;
	m_eyeangle = CIniFile::m_pCIniFile->GetValueDouble("3D", "eyeangle");
	m_eyeoffset = CIniFile::m_pCIniFile->GetValueDouble("3D", "eyeoffset");
	m_pScene = new COpenGLScene();
}

COpenGLFunctions::~COpenGLFunctions()
{	TRACE("COpenGLFunctions::~COpenGLFunctions()\n");
	HGLRC   hrc;
	hrc = ::wglGetCurrentContext();
	::wglMakeCurrent(NULL,  NULL);
	if (hrc)	::wglDeleteContext(hrc);
	//following linies added 20.12.2007
	POSITION pos = m_lpWnd.GetHeadPosition();
	OpenGLWnd wnd;
	for(int i = 1; i <= m_lpWnd.GetCount(); i++)
	{
		wnd = m_lpWnd.GetNext(pos);
		if (wnd.m_pDC)	delete wnd.m_pDC;
	}
	delete(m_pScene); 
}

//following method change by S.Wei 13.12.2007, last change 17.02.2008
void COpenGLFunctions::DrawScene(void)
{
	CMainFrame * mf = (CMainFrame *)IP_GET("m_pCMainFrame");
	if(GetWindowLong(mf->m_View2.m_hWnd, GWL_STYLE) & WS_VISIBLE)
	{//die Zweitanzeige ist sichtbar
		float pos = m_pScene->m_fViewPosition[0];
		float angle = m_pScene->m_fViewAngle[0];
		// Blickwinkel berechnen
		float eyeangle = EyeAngle(m_pScene->m_fScenePosition, m_pScene->m_fViewPosition,m_eyeoffset);
		
		//1.Anzeige
		//Augpunkt verschieben
		m_pScene->m_fViewPosition[0] = pos + m_eyeoffset;
		//Blickwinkel drehen
		m_pScene->m_fViewAngle[0] = angle + eyeangle;
		TrimValue(m_pScene->m_fViewAngle[0],0.0,360.0);
		
		//Szene zeichnen
		protectedDrawScene(&(mf->m_View1));
		
		//2.Anzeige
		//Augpunkt verschieben und Blickwinkel drehen
		m_pScene->m_fViewPosition[0] = pos - m_eyeoffset;
		m_pScene->m_fViewAngle[0] = angle - eyeangle;
		TrimValue(m_pScene->m_fViewAngle[0],0.0,360.0);

		//Szene zeichnen
		protectedDrawScene(&(mf->m_View2));
		
		//zurck setzen
		m_pScene->m_fViewPosition[0] = pos;
		m_pScene->m_fViewAngle[0] = angle;
		TrimValue(m_pScene->m_fViewAngle[0],0.0,360.0);
	}
	else
	{//Einzelanzeige
		
		protectedDrawScene(&(mf->m_wndView));
	}

}

void COpenGLFunctions::protectedDrawScene(CWnd * pWnd)
{
	makeCurrent(pWnd);
	ReSizeView();
	m_pScene->Draw();
	glFinish();
	SwapBuffers(wglGetCurrentDC());
}

bool COpenGLFunctions::SetupPixelFormat(void)
{	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
		1,                              // version number
		PFD_DRAW_TO_WINDOW |            // support window
		  PFD_SUPPORT_OPENGL |          // support OpenGL
		  PFD_DOUBLEBUFFER,             // double buffered
		PFD_TYPE_RGBA,                  // RGBA type
		24,                             // 24-bit color depth
		0, 0, 0, 0, 0, 0,               // color bits ignored
		0,                              // no alpha buffer
		0,                              // shift bit ignored
		0,                              // no accumulation buffer
		0, 0, 0, 0,                     // accum bits ignored
		32,                             // 32-bit z-buffer
		0,                              // no stencil buffer
		0,                              // no auxiliary buffer
		PFD_MAIN_PLANE,                 // main layer
		0,                              // reserved
		0, 0, 0                         // layer masks ignored
	};
	int pixelformat;
	if ((pixelformat = ChoosePixelFormat(m_hDC/*m_pDC->GetSafeHdc()*/, &pfd)) == 0 )
	{	AfxMessageBox("ChoosePixelFormat failed");
		return FALSE;
	}
	if (SetPixelFormat(m_hDC/*m_pDC->GetSafeHdc()*/, pixelformat, &pfd) == FALSE)
	{	AfxMessageBox("SetPixelFormat failed");
		return FALSE;
	}
	return TRUE;
}

void COpenGLFunctions::Init(CWnd * pWnd)
{
	m_pCOpenGLFunctions=this;
	
	TRACE("COpenGLFunctions::Init()\n");
	PIXELFORMATDESCRIPTOR pfd;
	int         n;

	createNewView(pWnd);
	
	n = ::GetPixelFormat(m_hDC);
	::DescribePixelFormat(m_hDC, n, sizeof(pfd), &pfd);
	m_hRC = wglCreateContext(m_hDC);
	wglMakeCurrent(m_hDC, m_hRC);

	InitView();
	m_pScene->Init();
}

//following method added by S.Wei, 13.12.2007
void COpenGLFunctions::createNewView(CWnd * pWnd)
{
	m_hWnd = pWnd->m_hWnd;
	m_pDC = new CClientDC(pWnd);
	m_hDC=m_pDC->GetSafeHdc();	//wenn vorher "*.m_pDC = new CClientDC(this);"
	ASSERT(m_pDC != NULL);
	ASSERT(m_hDC != NULL);
	
	if (!SetupPixelFormat())
	{	
		AfxMessageBox("COpenGLFunctions::Init()\n\nFehler bei SetupPixelFormat().",MB_OK+MB_ICONERROR,0);
		return;
	}

	//zur Liste hinzufgen
	OpenGLWnd newWnd;
	newWnd.m_hDC = m_hDC;
	newWnd.m_hWnd = m_hWnd;
	newWnd.m_pDC = m_pDC;
    m_lpWnd.AddTail(newWnd);
}

//following method added by S.Wei, 13.12.2007
void COpenGLFunctions::makeCurrent(CWnd * pWnd)
{
	int m = m_lpWnd.GetCount();
	if(m > 0)
	{//pWnd finden lassen
		POSITION pos = m_lpWnd.GetHeadPosition();
		OpenGLWnd wnd;
		while(m > 0)
		{
			wnd = m_lpWnd.GetNext(pos);
			if(wnd.m_hWnd == pWnd->m_hWnd)
			{//pWnd als aktuelle Ansicht setzen
				m_hWnd = wnd.m_hWnd;
				m_pDC = wnd.m_pDC;
				m_hDC = wnd.m_hDC;
				break;
			}
			m--;
		}
		if(m == 0)
		{//kein passendes Object gefunden
			createNewView(pWnd);
		}
	}
	else
	{//keine Elemente in der Liste
		createNewView(pWnd);
	}
	wglMakeCurrent(m_hDC, m_hRC);
}

void COpenGLFunctions::ReSizeView()
{
	CRect rect;	
	CWnd* pWnd = m_pDC->GetWindow();
	pWnd->GetClientRect(rect);
	
	int cx=rect.Width();
	int cy=rect.Height();
	if(cy > 0)		//d.h. wenigstens eine Zeile muss es noch geben
	{	glViewport(0, 0, cx, cy);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(45.0f, (GLfloat)cx/(GLfloat)cy, 0.2f,400.0f);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}
}

void COpenGLFunctions::InitView(void)
{
	TRACE("COpenGLFunctions::InitView()\n");
	
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);				// Black Background
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glClearDepth(1.0f);									// Depth Buffer Setup
	glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do

	glEnable(GL_COLOR_MATERIAL);
	
 	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	// Really Nice Perspective Calculations	//
// 	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_FASTEST);	// Really Nice Perspective Calculations	//
// 	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_DONT_CARE);// Really Nice Perspective Calculations	//

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glEnable(GL_POINT_SMOOTH);
//	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POLYGON_SMOOTH);

	glEnable(GL_CULL_FACE);
	glPolygonMode(GL_FRONT,GL_FILL);
	glPolygonMode(GL_BACK,GL_FILL);

	
}

bool COpenGLFunctions::IncludeSTLGeometry(CString filename)
{
	CString root="Geometrie";
	CString path=root+"/"+filename;
	CFile file;
	CFileException e;
	CFileStatus status;
	char buffer[100];
	unsigned int BytesRead;
	long MaxVertexCount;
	float normal[3]={0,0,0};
	float vertex[3][3];
	int n,v,k,t;
	float scale=10;
	
	if (!file.GetStatus(path,status))
	{	AfxMessageBox("COpenGLFunctions::IncludeSTLGeometry()\n\nProblem mit Datei: \""+filename+"\"\nim Verzeichnis: \""+root+"\".\n\nFr dieses Objekt werden keine Geometriedaten verwendet.",MB_OK+MB_ICONERROR,0);
		return false;
	};
	if( !file.Open( path, CFile::modeRead, &e ) )
	{	AfxMessageBox("COpenGLFunctions::IncludeSTLGeometry()\n\nProblem mit Datei:"+filename+"\nim Verzeichnis"+root+".\n\nFr dieses Objekt werden keine Geometriedaten verwendet.",MB_OK+MB_ICONERROR,0);
		return false;
	};
    	
	memset(buffer, 0, 100);
	file.SeekToBegin();

	BytesRead = file.Read(buffer, 80);
//	CString title=buffer;
//	title.TrimRight();

	BytesRead = file.Read(buffer, 4);
    memcpy(&MaxVertexCount,&buffer,4);

	TRACE("load bin-data (%d vertexes) from %s ... ",MaxVertexCount,path);
    
	for (t=0;t<=MaxVertexCount;t++)
	{	for (k=0;k<=2;k++)
		{	BytesRead = file.Read(buffer, 4);
			memcpy(&normal[k],&buffer,4);
		};
		for (v=0;v<=2;v++)
		{	for (k=0;k<=2;k++)
			{	BytesRead = file.Read(buffer, 4);
				memcpy(&vertex[v][k],&buffer,4);
				vertex[v][k]/=scale;
			}
		};
		BytesRead = file.Read(buffer, 2);
		glBegin(GL_POLYGON);
			glNormal3d(normal[0],normal[1],normal[2]);
			for (n=0;n<=2;n++) glVertex3d(vertex[n][0],vertex[n][1],vertex[n][2]);
		glEnd();

	};
	TRACE("successfull.\n",MaxVertexCount,path);
	file.Close();
	return true;
}