//////////////////////////////////////////////////////////////////////
// OpenGLScene.cpp: Schnittstelle fr die Klasse COpenGLScene.		//
// Written by Matthias Weber	wmat@mbv.tu-chemnitz.de  02/2003	//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// COpenGLScene.cpp: COpenGLScen-Klasse								//
//																	//
// Bedienungsanleitung:			erstellt von Weber Matthias 03/2003	//
//																	//
// ----------------------------------------------------------------	//

#include "stdafx.h"
#include "gl\gl.h"			//OpenGL - Standartbefehle	z.B. glVertex3d(), ...
#include "gl\glu.h"			//OpenGL - Utilities		z.B. gluPerspective(), ...
#include "gl\glaux.h"		//OpenGL - Hilfsbibliothek	z.B. auxSolidSphere(), ...
#include "math.h"
#include "Servo.h"
#include "Plattform.h"
#include "resource.h"
#include "IniFile.h"
#include "OpenGLObject.h"
#include "OpenGLLight.h"
#include "HexMotion.h"
#include "OpenGLScene.h"
#include "DlgStatus.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//COpenGLScene* COpenGLScene::m_pCOpenGLScene;				//statischer pointer auf instanz 

COpenGLScene::COpenGLScene()
{	
	TRACE("COpenGLScene::COpenGLScene()\n");
	//m_pCOpenGLScene=this;
	
	m_nMouseUsage=MU_NO_USAGE;
	m_bShowAllMarkers=false;
	m_bLight=false;
}

COpenGLScene::~COpenGLScene()
{	
	CIniFile* INI = CIniFile::m_pCIniFile;
	COpenGLObject* pObject;
	COpenGLLight* pLight;
	CServo* pServo;
	CPlattform* pPlattform;

	char buffer[50];
	POSITION pos;
	float fTemp[4];
	CString strTemp;
	int n;
	int nObjectNumber;

	TRACE("COpenGLScene::~COpenGLScene()\n");
		
	pos=COpenGLObject::m_pObjectList.GetHeadPosition();
	nObjectNumber=0;
	while (pos)			// genutzte objekte in INI eintragen
	{	pObject=COpenGLObject::m_pObjectList.GetNext(pos);
		if (pObject->GetStatus()=="passiv")
		{	nObjectNumber++;
			sprintf(buffer, "object%d",nObjectNumber);		
			INI->SetValue("OBJECTS_PASSIV",buffer,pObject->GetName());
			sprintf(buffer, "%s",pObject->GetName());		
			pObject->GetPosition(fTemp);
			 INI->SetValue(buffer,"position",fTemp,true);
			pObject->GetOrientation(fTemp);
			 INI->SetValue(buffer,"orientation",fTemp,true);
			pObject->GetPositionOffset(fTemp);
			 INI->SetValue(buffer,"position_offset",fTemp,true);
			pObject->GetOrientationOffset(fTemp);
			 INI->SetValue(buffer,"orientation_offset",fTemp,true);
			pObject->GetColor(fTemp);
			 INI->SetValue(buffer,"color",fTemp,true); 
			 INI->SetValue(buffer,"color_alpha",fTemp[3],true);	
			strTemp=pObject->GetGeometrieFile();
			 INI->SetValue(buffer,"geometrie_file",strTemp,true);	
			if (pObject->GetShowOrigin())
			 INI->SetValue(buffer,"show_origin",(int)1,true);
			 else
			 INI->SetValue(buffer,"show_origin",(int)0,true);
			if (pObject->GetShowGeometry())
			 INI->SetValue(buffer,"show_geometrie",(int)1,true);
			 else
			 INI->SetValue(buffer,"show_geometrie",(int)0,true);
			delete (pObject);
		};
	};
	bool LastObject=false;
	while (!LastObject)	// ungenutzte Objekte aus INI lschen
	{	nObjectNumber++;
		sprintf(buffer, "object%d",nObjectNumber);
		if (INI->IsValue("OBJECTS",buffer))
		{	INI->DeleteValue("OBJECTS",buffer);
			LastObject = false;
		}else LastObject = true;
	};
	//	COpenGLObject::m_pObjectList.RemoveAll();  



	//	delete (COpenGLLight::GetGLLight(GL_LIGHT0));
	pos=COpenGLLight::m_pLightList.GetHeadPosition();
	while (pos)
	{	pLight=COpenGLLight::m_pLightList.GetNext(pos);
		delete (pLight);
	};
	COpenGLLight::m_pLightList.RemoveAll();  



	//	delete (CServo::GetServo("...."));
	pos=CServo::m_pServoList.GetHeadPosition();
	while (pos)
	{	pServo=CServo::m_pServoList.GetNext(pos);
		if (pServo)
		{	//sprintf(buffer, "servo%d",pServo->GetServoNumber());		
			pServo->GetPosition(fTemp);
			INI->SetValue(pServo->GetName(),"position",fTemp,true);
			
			pServo->GetOrientation(fTemp);
			INI->SetValue(pServo->GetName(),"orientation",fTemp,true);
			
			INI->SetValue(pServo->GetName(),"angle",pServo->GetAngle(),true);
			INI->SetValue(pServo->GetName(),"startangle",pServo->GetStartAngle(),true);
			INI->SetValue(pServo->GetName(),"armlength",pServo->GetArmLength(),true);
			INI->SetValue(pServo->GetName(),"linklength",pServo->GetLinkLength(),true);
			INI->SetValue(pServo->GetName(),"anglefactor",pServo->GetAngleCorrectionFactor(),true);
			INI->SetValue(pServo->GetName(),"usedangle",pServo->GetUsedAngle(),true);
		};
		sprintf(buffer, "servo%d_housing",pServo->GetServoNumber());
		pObject=COpenGLObject::GetObject(buffer);
		if (pObject)
		{	pObject->GetPositionOffset(fTemp);
			 INI->SetValue(buffer,"position_offset",fTemp,true);
			pObject->GetOrientationOffset(fTemp);
			 INI->SetValue(buffer,"orientation_offset",fTemp,true);
			pObject->GetColor(fTemp);
			 INI->SetValue(buffer,"color",fTemp,true); 
			 INI->SetValue(buffer,"color_alpha",fTemp[3],true);	
			strTemp=pObject->GetGeometrieFile();
			 INI->SetValue(buffer,"geometrie_file",strTemp,true);	
			if (pObject->GetShowOrigin())
			 INI->SetValue(buffer,"show_origin",(int)1,true);
			 else
			 INI->SetValue(buffer,"show_origin",(int)0,true);
			if (pObject->GetShowGeometry())
			 INI->SetValue(buffer,"show_geometrie",(int)1,true);
			 else
			 INI->SetValue(buffer,"show_geometrie",(int)0,true);
		};
		sprintf(buffer, "servo%d_angle",pServo->GetServoNumber());
		pObject=COpenGLObject::GetObject(buffer);
		if (pObject)
		{	pObject->GetPositionOffset(fTemp);
			 INI->SetValue(buffer,"position_offset",fTemp,true);
			pObject->GetOrientationOffset(fTemp);
			 INI->SetValue(buffer,"orientation_offset",fTemp,true);
			pObject->GetColor(fTemp);
			 INI->SetValue(buffer,"color",fTemp,true); 
			 INI->SetValue(buffer,"color_alpha",fTemp[3],true);	
			strTemp=pObject->GetGeometrieFile();
			 INI->SetValue(buffer,"geometrie_file",strTemp,true);	
			if (pObject->GetShowOrigin())
			 INI->SetValue(buffer,"show_origin",(int)1,true);
			 else
			 INI->SetValue(buffer,"show_origin",(int)0,true);
			if (pObject->GetShowGeometry())
			 INI->SetValue(buffer,"show_geometrie",(int)1,true);
			 else
			 INI->SetValue(buffer,"show_geometrie",(int)0,true);
		};
		delete (pServo);
	};
	CServo::m_pServoList.RemoveAll();  

	
	pPlattform=CPlattform::m_pCPlattform;
	if (pPlattform)
	{	pPlattform->GetPosition(fTemp);
		INI->SetValue("plattform","position",fTemp,true);
		pPlattform->GetOrientation(fTemp);
		INI->SetValue("plattform","orientation",fTemp,true);
		
		pPlattform->GetLinkPointLokal(1,fTemp);
		INI->SetValue("plattform","linkpoint1",fTemp,true);
		pPlattform->GetLinkPointLokal(2,fTemp);
		INI->SetValue("plattform","linkpoint2",fTemp,true);
		pPlattform->GetLinkPointLokal(3,fTemp);
		INI->SetValue("plattform","linkpoint3",fTemp,true);
		pPlattform->GetLinkPointLokal(4,fTemp);
		INI->SetValue("plattform","linkpoint4",fTemp,true);
		pPlattform->GetLinkPointLokal(5,fTemp);
		INI->SetValue("plattform","linkpoint5",fTemp,true);
		pPlattform->GetLinkPointLokal(6,fTemp);
		INI->SetValue("plattform","linkpoint6",fTemp,true);
		
		INI->SetValue("plattform","incr_translation",pPlattform->m_dIncrTranslation,true);
		INI->SetValue("plattform","incr_rotation",pPlattform->m_dIncrRotation,true);
		
		pObject=COpenGLObject::GetObject("plattform_object");
		if (pObject)
		{	pObject->GetPositionOffset(fTemp);
			 INI->SetValue(pObject->GetName(),"position_offset",fTemp,true);
			pObject->GetOrientationOffset(fTemp);
			 INI->SetValue(pObject->GetName(),"orientation_offset",fTemp,true);
			pObject->GetColor(fTemp);
			 INI->SetValue(pObject->GetName(),"color",fTemp,true); 
			 INI->SetValue(pObject->GetName(),"color_alpha",fTemp[3],true);	
			strTemp=pObject->GetGeometrieFile();
			 INI->SetValue(pObject->GetName(),"geometrie_file",strTemp,true);	
			if (pObject->GetShowOrigin())
			 INI->SetValue(pObject->GetName(),"show_origin",(int)1,true);
			 else
			 INI->SetValue(pObject->GetName(),"show_origin",(int)0,true);
			if (pObject->GetShowGeometry())
			 INI->SetValue(pObject->GetName(),"show_geometrie",(int)1,true);
			 else
			 INI->SetValue(pObject->GetName(),"show_geometrie",(int)0,true);
		};
		delete (pPlattform);
	};
	
	for (n=0;n<3;n++) fTemp[n]=m_fViewPosition[n];
	 INI->SetValue("SCENE_VP_A","viewposition",fTemp,true);
	for (n=0;n<3;n++) fTemp[n]=m_fSceneAngle[n];
	 INI->SetValue("SCENE_VP_A","sceneangle",fTemp,true);
	for (n=0;n<3;n++) fTemp[n]=m_fScenePosition[n];
	 INI->SetValue("SCENE_VP_A","sceneposition",fTemp,true);

}

void COpenGLScene::Draw()
{
	glClearColor(m_fColor_SceneBackground[0],m_fColor_SceneBackground[1],m_fColor_SceneBackground[2], 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();
	DrawLights();
	
	//-------Blickwinkel--------------------------------------------
	glTranslated(-m_fViewPosition[0], -m_fViewPosition[1], -m_fViewPosition[2]);
	
	//following 3 lines added by S.Wei
	glRotated(m_fViewAngle[2],0.0,0.0,1.0);
	glRotated(m_fViewAngle[1],1.0,0.0,0.0);
	glRotated(m_fViewAngle[0],0.0,1.0,0.0);
	//----------------------------------

	glRotatef(m_fSceneAngle[2],0.0,0.0,1.0);
	glRotatef(m_fSceneAngle[1],1.0,0.0,0.0);
	glRotatef(m_fSceneAngle[0],0.0,1.0,0.0);
	
	glTranslatef(-m_fScenePosition[0],-m_fScenePosition[1],-m_fScenePosition[2]);
	//--------------------------------------------------------------
	
	DrawFixedLights();

	glDisable(GL_LIGHTING);
	if (m_bLight)	SetMaterial();
	DrawMarkers();
	DrawGrids();
	if (m_bLight)	glEnable(GL_LIGHTING);

	
	if(m_nShowSettings & SHOW_SERVO_DEBUG)
		DrawServoDebug();	
	if(m_nShowSettings & SHOW_PLATTFORM_DEBUG)
		DrawPlattformDebug();
	if(m_nShowSettings & SHOW_LINK_DEBUG)
		DrawLinkDebug();

	glDepthMask(false);
	DrawObjects();
	glDepthMask(true);
	DrawObjects();

		
}

void COpenGLScene::Init()
{	CIniFile* INI = CIniFile::m_pCIniFile;
	CDlgStatus* pDlgStatus = CDlgStatus::m_pCDlgStatus;
	
	if (pDlgStatus) pDlgStatus->Show();
	if (pDlgStatus) pDlgStatus->SetTextA("Initialisiere Szene ...");
	if (pDlgStatus) pDlgStatus->SetTextB("");
	
	TRACE("COpenGLScene::Init()\n");
	if (pDlgStatus) 
		pDlgStatus->SetTextB("lese INI-File");					//Statusmeldung ausgeben
	InitVariables();											//Variablen mit Daten aus dem INI-File initialisieren 
	
	if (pDlgStatus) pDlgStatus->SetTextB("lese Viewpoints");	//Statusmeldung ausgeben
	InitViewPoints();											//Viewpoint-Variablen mit Daten aus dem INI-File initialisieren
	
	if (pDlgStatus) 
		pDlgStatus->SetTextB("setze Viewpoint");				//Statusmeldung ausgeben
	SetViewPoint(0);											//Aktuellen Viewpoint setzen 
	
	if (pDlgStatus) 
		pDlgStatus->SetTextB("lade Grid's");					//Statusmeldung ausgeben
	InitGrids();												//Grid-Display-Listen erstellen
	
	if (pDlgStatus) 
		pDlgStatus->SetTextB("lade Maus-Marker");				//Statusmeldung ausgeben
	InitMarkers();												//Marker-Display-Listen erstellen	

	if (pDlgStatus) pDlgStatus->SetTextB("lade Beleuchtung");
	if (INI->IsKey("GL_LIGHT0"))	
		AddLight(GL_LIGHT0);
	if (INI->IsKey("GL_LIGHT1"))	
		AddLight(GL_LIGHT1);
	if (INI->IsKey("GL_LIGHT2"))	
		AddLight(GL_LIGHT2);
	if (INI->IsKey("GL_LIGHT3"))	
		AddLight(GL_LIGHT3);
	if (INI->IsKey("GL_LIGHT4"))	
		AddLight(GL_LIGHT4);
	if (INI->IsKey("GL_LIGHT5"))	
		AddLight(GL_LIGHT5);
	if (INI->IsKey("GL_LIGHT6"))	
		AddLight(GL_LIGHT6);
	if (INI->IsKey("GL_LIGHT7"))	
		AddLight(GL_LIGHT7);
	
	if (pDlgStatus) pDlgStatus->SetTextA("InitObjects()...");
	InitObjects();
	
	if (pDlgStatus) pDlgStatus->SetTextA("InitPlattform()...");
	InitPlattform();

	if (pDlgStatus) pDlgStatus->SetTextA("InitServos()...");
	InitServos();
	
	if (pDlgStatus) pDlgStatus->Hide();							//Statusfenster ausblenden
}

void COpenGLScene::InitGrids()
{
	double x;//,y,z;
	double Size=20.0;
	int Intervalls=20;
	double Diff=Size/Intervalls;
	
	// Gitternetz fr XZ-Ebene
	if (!glIsList(m_nSceneGridList[0]))	m_nSceneGridList[0] = glGenLists(1);
	glNewList(m_nSceneGridList[0], GL_COMPILE);
		glPushMatrix();
			glColor3d(m_fColor_Grid[0][0],m_fColor_Grid[0][1],m_fColor_Grid[0][2]);	
			glLineWidth(1.0);
			glBegin(GL_LINES);
/*			  for (x=-Size/2;x<=Size/2;x+=Diff)
				for (z=-Size/2;z<Size/2;z+=Diff)
				{	glVertex3d(x,0.0,z);
					glVertex3d(x,0.0,z+Diff);
					glVertex3d(z,0.0,x);
					glVertex3d(z+Diff,0.0,x);
				};
*/			  for (x=-Size/2;x<=Size/2;x+=Diff)
				{	glVertex3d(-Size/2,0.0,x);
					glVertex3d(+Size/2,0.0,x);
					glVertex3d(x,0.0,-Size/2);
					glVertex3d(x,0.0,+Size/2);
				};
			glEnd();
		glPopMatrix();
	glEndList();
	// Gitternetz fr XY-Ebene
	if (!glIsList(m_nSceneGridList[1]))	m_nSceneGridList[1] = glGenLists(1);
	glNewList(m_nSceneGridList[1], GL_COMPILE);
		glPushMatrix();
			glColor3d(m_fColor_Grid[1][0],m_fColor_Grid[1][1],m_fColor_Grid[1][2]);	
			glLineWidth(1.0);
			glBegin(GL_LINES);
/*			 for (x=-Size/2;x<=Size/2;x+=Diff)
				for (y=-Size/2;y<Size/2;y+=Diff)
				{	glVertex3d(x,y,0.0);
					glVertex3d(x,y+Diff,0.0);
					glVertex3d(y,x,0.0);
					glVertex3d(y+Diff,x,0.0);
				};
*/			  for (x=-Size/2;x<=Size/2;x+=Diff)
				{	glVertex3d(-Size/2,x,0.0);
					glVertex3d(+Size/2,x,0.0);
					glVertex3d(x,-Size/2,0.0);
					glVertex3d(x,+Size/2,0.0);
				};
			glEnd();
		glPopMatrix();
	glEndList();
	// Gitternetz fr YZ-Ebene
	if (!glIsList(m_nSceneGridList[2]))	m_nSceneGridList[2] = glGenLists(1);
	glNewList(m_nSceneGridList[2], GL_COMPILE);
		glPushMatrix();
			glColor3d(m_fColor_Grid[2][0],m_fColor_Grid[2][1],m_fColor_Grid[2][2]);	
			glLineWidth(1.0);
			glBegin(GL_LINES);
/*			 for (y=-Size/2;y<=Size/2;y+=Diff)
				for (z=-Size/2;z<Size/2;z+=Diff)
				{	glVertex3d(0.0,y,z);
					glVertex3d(0.0,y,z+Diff);
					glVertex3d(0.0,z,y);
					glVertex3d(0.0,z+Diff,y);
				};
*/			  for (x=-Size/2;x<=Size/2;x+=Diff)
				{	glVertex3d(0.0,-Size/2,x);
					glVertex3d(0.0,+Size/2,x);
					glVertex3d(0.0,x,-Size/2);
					glVertex3d(0.0,x,+Size/2);
				};

			glEnd();
		glPopMatrix();
	glEndList();
}

void COpenGLScene::InitMarkers()
{
	double n;
	double radius=0.6;
	
	//TRANSLATION-MARKER
	radius=0.1;
	if (!glIsList(m_nSceneMarkerList[0])) m_nSceneMarkerList[0] = glGenLists(1);
	glNewList(m_nSceneMarkerList[0], GL_COMPILE);
		glPushMatrix();
			glLoadIdentity();
			glTranslated(0.0,0.0,-3.0);
			glColor3d(m_fColor_Markers[0],m_fColor_Markers[1],m_fColor_Markers[2]);	
			glLineWidth(1.0);
			glBegin(GL_LINE_STRIP);
				for (n=0.0;n<=360.0;n+=20.0)
					glVertex3d(radius*sin(n*CServo::piover180),radius*cos(n*CServo::piover180),0.0);
			glEnd();

			glBegin(GL_LINES);
				glVertex3d(-1.0,0.0,0.0);
				 glVertex3d(-radius,0.0,0.0);
				glVertex3d(1.0,0.0,0.0);
				 glVertex3d(radius,0.0,0.0);
				glVertex3d(-1.0,0.0,0.0);
				 glVertex3d(-0.8,0.02,0.0);
				glVertex3d(-1.0,0.0,0.0);
				 glVertex3d(-0.8,-0.02,0.0);
				glVertex3d(1.0,0.0,0.0);
				 glVertex3d(0.8,0.02,0.0);
				glVertex3d(1.0,0.0,0.0);
				 glVertex3d(0.8,-0.02,0.0);

				glVertex3d(0.0,-1.0,0.0);
				 glVertex3d(0.0,-radius,0.0);
				glVertex3d(0.0,1.0,0.0);
				 glVertex3d(0.0,radius,0.0);
				
				glVertex3d(0.0,-1.0,0.0);
				 glVertex3d(0.02,-0.8,0.0);
				glVertex3d(0.0,-1.0,0.0);
				 glVertex3d(-0.02,-0.8,0.0);
				glVertex3d(0.0,1.0,0.0);
				 glVertex3d(0.02,0.8,0.0);
				glVertex3d(0.0,1.0,0.0);
				 glVertex3d(-0.02,0.8,0.0);
			glEnd();
			glPointSize(3.0);
			glBegin(GL_POINTS);
				glVertex3d(0.0,0.0,0.0);	 
			glEnd();
		glPopMatrix();
	glEndList();
	
	// TRANSLATION_INTO-MARKER
	if (!glIsList(m_nSceneMarkerList[1])) m_nSceneMarkerList[1] = glGenLists(1);
	glNewList(m_nSceneMarkerList[1], GL_COMPILE);
		glPushMatrix();
			glLoadIdentity();
			glTranslated(0.0,0.0,-3.0);
			glColor3d(m_fColor_Markers[0],m_fColor_Markers[1],m_fColor_Markers[2]);	
			glLineWidth(1.0);
			glBegin(GL_LINES);
				glVertex3d(-0.5,-0.5,1.0);	 glVertex3d(-0.1,-0.1,0.2);
				glVertex3d(-0.5,-0.5,1.0);	 glVertex3d(-0.45,-0.35,0.8);
				glVertex3d(-0.5,-0.5,1.0);	 glVertex3d(-0.35,-0.45,0.8);
				glVertex3d(-0.1,-0.1,0.2);	 glVertex3d(-0.13,-0.16,0.3);
				glVertex3d(-0.1,-0.1,0.2);	 glVertex3d(-0.16,-0.13,0.3);

				glVertex3d(0.5,-0.5,1.0);	 glVertex3d(0.1,-0.1,0.2);
				glVertex3d(0.5,-0.5,1.0);	 glVertex3d(0.45,-0.35,0.8);
				glVertex3d(0.5,-0.5,1.0);	 glVertex3d(0.35,-0.45,0.8);
				glVertex3d(0.1,-0.1,0.2);	 glVertex3d(0.13,-0.16,0.3);
				glVertex3d(0.1,-0.1,0.2);	 glVertex3d(0.16,-0.13,0.3);

				glVertex3d(-0.5,0.5,1.0);	 glVertex3d(-0.1,0.1,0.2);
				glVertex3d(-0.5,0.5,1.0);	 glVertex3d(-0.45,0.35,0.8);
				glVertex3d(-0.5,0.5,1.0);	 glVertex3d(-0.35,0.45,0.8);
				glVertex3d(-0.1,0.1,0.2);	 glVertex3d(-0.13,0.16,0.3);
				glVertex3d(-0.1,0.1,0.2);	 glVertex3d(-0.16,0.13,0.3);
				
				glVertex3d(0.5,0.5,1.0);	 glVertex3d(0.1,0.1,0.2);
				glVertex3d(0.5,0.5,1.0);	 glVertex3d(0.45,0.35,0.8);
				glVertex3d(0.5,0.5,1.0);	 glVertex3d(0.35,0.45,0.8);
				glVertex3d(0.1,0.1,0.2);	 glVertex3d(0.13,0.16,0.3);
				glVertex3d(0.1,0.1,0.2);	 glVertex3d(0.16,0.13,0.3);

			glEnd();
		glPopMatrix();
	glEndList();
	
	// ZOOM-MARKER //veraltet seit 02.01.2008
	radius=0.6;
	if (!glIsList(m_nSceneMarkerList[2])) m_nSceneMarkerList[2] = glGenLists(1);
	glNewList(m_nSceneMarkerList[2], GL_COMPILE);
		glPushMatrix();
			glLoadIdentity();
			glTranslated(0.0,0.0,-3.0);
			glColor3d(m_fColor_Markers[0],m_fColor_Markers[1],m_fColor_Markers[2]);	
			glLineWidth(1.0);
			glBegin(GL_LINE_STRIP);
				for (n=0.0;n<=360.0;n+=10.0)
					glVertex3d(radius*sin(n*CServo::piover180),radius*cos(n*CServo::piover180),0.0);
			glEnd();
			glBegin(GL_LINES);
				glVertex3d(radius*sin(47.0*CServo::piover180),-radius*cos(47.0*CServo::piover180),0.0);
				glVertex3d(1.1,-1.0,0.0);
				glVertex3d(radius*sin(43.0*CServo::piover180),-radius*cos(43.0*CServo::piover180),0.0);
				glVertex3d(1.0,-1.1,0.0);	
			glEnd();
		glPopMatrix();
	glEndList();

	
	// ROTATION-MARKER
	radius=0.6;
	if (!glIsList(m_nSceneMarkerList[3])) m_nSceneMarkerList[3] = glGenLists(1);
	glNewList(m_nSceneMarkerList[3], GL_COMPILE);
		glPushMatrix();
			glLoadIdentity();
			glTranslated(0.0,0.0,-3.0);
			glColor3d(m_fColor_Markers[0],m_fColor_Markers[1],m_fColor_Markers[2]);	
			glLineWidth(1.0);
			glBegin(GL_LINE_STRIP);
				for (n=0.0;n<=360.0;n+=10.0)
					glVertex3d(radius*sin(n*CServo::piover180),radius*cos(n*CServo::piover180),0.0);
			glEnd();
			glBegin(GL_LINES);
				glVertex3d(radius,0.0,0.0);	 glVertex3d(1.0,0.0,0.0);
				glVertex3d(-radius,0.0,0.0); glVertex3d(-1.0,0.0,0.0);
				glVertex3d(radius,0.0,0.0);	 glVertex3d(1.0,0.0,0.0);
				glVertex3d(0.0,radius,0.0);	 glVertex3d(0.0,1.0,0.0);
				glVertex3d(0.0,-radius,0.0); glVertex3d(0.0,-1.0,0.0);
				
				glVertex3d(0.0,0.2,0.0);	 glVertex3d(0.0,-0.2,0.0);
				glVertex3d(0.2,0.0,0.0);	 glVertex3d(-0.2,0.0,0.0);
			glEnd();
		glPopMatrix();
	glEndList();

}

void COpenGLScene::InitViewPoints()
{	
	CIniFile* INI = CIniFile::m_pCIniFile;

	INI->GetValueTrFloat("SCENE_VP_A","viewposition"	,m_fViewPoint[0]+0);
	INI->GetValueTrFloat("SCENE_VP_A","sceneangle"		,m_fViewPoint[0]+3);
	INI->GetValueTrFloat("SCENE_VP_A","sceneposition"	,m_fViewPoint[0]+6);
	INI->GetValueTrFloat("SCENE_VP_A","viewangle"		,m_fViewPoint[0]+9);
	
	INI->GetValueTrFloat("SCENE_VP_B","viewposition"	,m_fViewPoint[1]+0);
	INI->GetValueTrFloat("SCENE_VP_B","sceneangle"		,m_fViewPoint[1]+3);
	INI->GetValueTrFloat("SCENE_VP_B","sceneposition"	,m_fViewPoint[1]+6);
	INI->GetValueTrFloat("SCENE_VP_A","viewangle"		,m_fViewPoint[0]+9);
	
	INI->GetValueTrFloat("SCENE_VP_C","viewposition"	,m_fViewPoint[2]+0);
	INI->GetValueTrFloat("SCENE_VP_C","sceneangle"		,m_fViewPoint[2]+3);
	INI->GetValueTrFloat("SCENE_VP_C","sceneposition"	,m_fViewPoint[2]+6);
	INI->GetValueTrFloat("SCENE_VP_A","viewangle"		,m_fViewPoint[0]+9);
	
	INI->GetValueTrFloat("SCENE_VP_D","viewposition"	,m_fViewPoint[3]+0);
	INI->GetValueTrFloat("SCENE_VP_D","sceneangle"		,m_fViewPoint[3]+3);
	INI->GetValueTrFloat("SCENE_VP_D","sceneposition"	,m_fViewPoint[3]+6);
	INI->GetValueTrFloat("SCENE_VP_A","viewangle"		,m_fViewPoint[0]+9);
}

void COpenGLScene::SetViewPoint(int nVP)
{/*	m_fViewPosition[0]=0.0;		//Position des Augpunktes
	m_fViewPosition[1]=0.0;
	m_fViewPosition[2]=5.0;
	
	m_fViewAngle[0]=0.0;		//Drehen (pos= Kopf nach rechts drehen)
	m_fViewAngle[1]=0.0;		//Nicken (pos= Kopf nach unten nicken)
	m_fViewAngle[2]=0.0;		//Neigen (pos= Kopf nach rechts neigen)

	m_fScenePosition[0]=0.0;	//Position des Szenenmittelpunktes ind der Szene
	m_fScenePosition[1]=0.0;
	m_fScenePosition[2]=0.0;

	m_fSceneAngle[0]=-45.0;		//Drehung um y "rechte Hand"
	m_fSceneAngle[1]=45.0;		//Kippen um x
	m_fSceneAngle[2]=0.0;		//Neigen um z
*/
	m_fViewPosition[0]=m_fViewPoint[nVP][0];	//Position des Augpunktes
	m_fViewPosition[1]=m_fViewPoint[nVP][1];
	m_fViewPosition[2]=m_fViewPoint[nVP][2];
	
	m_fSceneAngle[0]=m_fViewPoint[nVP][3];		//Drehung um y "rechte Hand"
	m_fSceneAngle[1]=m_fViewPoint[nVP][4];		//Kippen um x
	m_fSceneAngle[2]=m_fViewPoint[nVP][5];		//Neigen um z
	
	m_fScenePosition[0]=m_fViewPoint[nVP][6];	//Position des Szenenmittelpunktes ind der Szene
	m_fScenePosition[1]=m_fViewPoint[nVP][7];
	m_fScenePosition[2]=m_fViewPoint[nVP][8];

	m_fViewAngle[0]=m_fViewPoint[nVP][9];	//Drehen (Kopf nach rechts drehen > 0)
	m_fViewAngle[1]=m_fViewPoint[nVP][10];	//Nicken (Kopf nach unten nicken  > 0)
	m_fViewAngle[2]=m_fViewPoint[nVP][11];	//Neigen (Kopf nach rechts neigen > 0)

}

void COpenGLScene::InitVariables()
{	
	CIniFile* INI = CIniFile::m_pCIniFile;
	m_nShowSettings=0;
	if (INI->GetValueInt("SCENE","grid_xz_show")>0) 
		m_nShowSettings|=SHOW_GRID_XZ;
	if (INI->GetValueInt("SCENE","grid_xy_show")>0) 
		m_nShowSettings|=SHOW_GRID_XY;
	if (INI->GetValueInt("SCENE","grid_yz_show")>0) 
		m_nShowSettings|=SHOW_GRID_YZ;
	INI->GetValueTrFloat("SCENE","grid_xz_color",m_fColor_Grid[0]);
	INI->GetValueTrFloat("SCENE","grid_xy_color",m_fColor_Grid[1]);
	INI->GetValueTrFloat("SCENE","grid_yz_color",m_fColor_Grid[2]);

	if (INI->GetValueInt("SCENE","servo_debug")>0)		
		m_nShowSettings|=SHOW_SERVO_DEBUG;
	if (INI->GetValueInt("SCENE","plattform_debug")>0)	
		m_nShowSettings|=SHOW_PLATTFORM_DEBUG;
	if (INI->GetValueInt("SCENE","link_debug")>0)		
		m_nShowSettings|=SHOW_LINK_DEBUG;

	if (INI->GetValueInt("SCENE","markers_show")>0)
		m_nShowSettings|=SHOW_MARKERS;
	INI->GetValueTrFloat("SCENE","markers_color",m_fColor_Markers);
	
	INI->GetValueTrFloat("SCENE","background_color",m_fColor_SceneBackground);
	
	m_bLight= (INI->GetValueInt("SCENE","lighting")>0);
}

void COpenGLScene::DrawLights()
{
	COpenGLLight* pLight;
	int n;
	
	for (n=GL_LIGHT0;n<=GL_LIGHT7;n++)
	{		
		pLight=COpenGLLight::GetGLLight(n);
		if (pLight) 
		{	if (!pLight->IsFixed())	
			{	pLight->DrawLight();
				if ((pLight->IsActivated())&&(pLight->m_bDrawSourcePoint))
				{	glPushMatrix();
						glDisable(GL_LIGHTING);
						glColor3d(1.0,0.0,0.0);
						glPointSize(20.0);
						glBegin(GL_POINTS);
							glVertex3f(pLight->m_fPosition[0],pLight->m_fPosition[1],pLight->m_fPosition[2]);
						glEnd();
						if (m_bLight) glEnable(GL_LIGHTING);
					glPopMatrix();
				};
			};
		}else
			glDisable(n);
	}	
}
void COpenGLScene::DrawFixedLights()
{
	COpenGLLight* pLight;
	int n;
	for (n=GL_LIGHT0;n<=GL_LIGHT7;n++)
	{		
		pLight=COpenGLLight::GetGLLight(n);
		
		if (pLight) 
		{	if (pLight->IsFixed())
			{	pLight->DrawLight();
				if ((pLight->IsActivated())&&(pLight->m_bDrawSourcePoint))
				{	glPushMatrix();
						glDisable(GL_LIGHTING);
						glColor3d(1.0,1.0,1.0);
						glPointSize(20.0);
						glBegin(GL_POINTS);
							glVertex3f(pLight->m_fPosition[0],pLight->m_fPosition[1],pLight->m_fPosition[2]);
						glEnd();
						if (m_bLight) glEnable(GL_LIGHTING);
					glPopMatrix();
				};
			};
		}else
			glDisable(n);
	}	
}


void COpenGLScene::DrawMarkers()
{
	if (m_bShowAllMarkers)
	{	if (glIsList(m_nSceneMarkerList[0])) glCallList(m_nSceneMarkerList[0]);	
		if (glIsList(m_nSceneMarkerList[1])) glCallList(m_nSceneMarkerList[1]);
		if (glIsList(m_nSceneMarkerList[2])) glCallList(m_nSceneMarkerList[2]);
		if (glIsList(m_nSceneMarkerList[3])) glCallList(m_nSceneMarkerList[3]);
	}

	if(m_nShowSettings & SHOW_MARKERS)
	{	if(m_nMouseUsage == MU_SCENE_TRANSLATE)
			if (glIsList(m_nSceneMarkerList[0])) glCallList(m_nSceneMarkerList[0]);
		if(m_nMouseUsage == MU_SCENE_TRANSLATE_INTO)
			if (glIsList(m_nSceneMarkerList[1])) glCallList(m_nSceneMarkerList[1]);		  
		if(m_nMouseUsage == MU_SCENE_ROTATE)
			if (glIsList(m_nSceneMarkerList[3])) glCallList(m_nSceneMarkerList[3]);
		if(( m_nMouseUsage == MU_SCENE_TRANSLATE_INTO)||
			(m_nMouseUsage == MU_SCENE_TRANSLATE)||
			(m_nMouseUsage == MU_SCENE_ROTATE))
		{	glPushMatrix();
				glLoadIdentity();
				glTranslated(-m_fViewPosition[0], -m_fViewPosition[1], -m_fViewPosition[2]);
				glColor3d(1.0,1.0,1.0);
				glPointSize(10.0);
				glEnable(GL_POINT_SMOOTH);
				glBegin(GL_POINTS);
					glVertex3d(0.0,0.0,0.0);	 
				glEnd();
			glPopMatrix();
		};
	}
}

void COpenGLScene::DrawGrids()
{
	if(m_nShowSettings & SHOW_GRID_XZ)
		if (glIsList(m_nSceneGridList[0]))	glCallList(m_nSceneGridList[0]);
	if(m_nShowSettings & SHOW_GRID_XY)	
		if (glIsList(m_nSceneGridList[1]))	glCallList(m_nSceneGridList[1]);
	if(m_nShowSettings & SHOW_GRID_YZ)	
		if (glIsList(m_nSceneGridList[2]))	glCallList(m_nSceneGridList[2]);
}

void COpenGLScene::DrawObjects()
{
	POSITION pos=COpenGLObject::m_pObjectList.GetHeadPosition();
	COpenGLObject* pObject;
	while (pos)
	{	
		pObject=COpenGLObject::m_pObjectList.GetNext(pos);
		glPushMatrix();
			pObject->Draw();
		glPopMatrix();
	};

}

void COpenGLScene::SetMaterial()
{	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	float mat_specular[]={1.0f,1.0f,1.0f,1.0f};
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	float mat_ambient[]={0.1f,0.1f,0.1f,1.0f};
	glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
	float mat_diffuse[]={0.5f,0.5f,0.5f,1.0f};
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
	
	glMateriali(GL_FRONT,GL_SHININESS,50);

}

void COpenGLScene::AddLight(int nGLLightNr)
{	COpenGLLight* pLight;
	ASSERT(nGLLightNr>=GL_LIGHT0);
	ASSERT(nGLLightNr<=GL_LIGHT7);
	if (!COpenGLLight::GetGLLight(nGLLightNr))
	{	pLight = new COpenGLLight(nGLLightNr);
		pLight->Init();
	}
	else
	{	/*
		char buffer[50];
		CString temp;
		sprintf(buffer, "GL_LIGHT%d",nGLLightNr-GL_LIGHT0);
		temp=buffer;
		AfxMessageBox("COpenGLScene::AddLight()\n\nFehler: "+temp+" ist schon vorhanden.\nAddLight() wird ignoriert.",MB_OK+MB_ICONERROR,0);*/
	}
}

void COpenGLScene::InitObjects()
{	COpenGLObject* pObject;
	CIniFile* INI = CIniFile::m_pCIniFile;
	CDlgStatus* pDlgStatus = CDlgStatus::m_pCDlgStatus;
	
	if (pDlgStatus) pDlgStatus->SetTextB("searching Objects...");

	float temp[4];
	CString ObjectName;
	CString GeometrieName;
	bool addobjects=true;
	
	char buffer[50];
		
	int n=0;
	while (addobjects)
	{	n+=1;
		sprintf(buffer, "object%d",n);

		ObjectName=INI->GetValue("OBJECTS_PASSIV",buffer);

		if (ObjectName.GetLength()>0)
		{	if (pDlgStatus) pDlgStatus->SetTextB("lade: \""+ObjectName+"\"");
			pObject = new COpenGLObject(ObjectName);
			
			INI->GetValueTrFloat(ObjectName,"position",temp);
			pObject ->SetPosition(temp);
			
			INI->GetValueTrFloat(ObjectName,"orientation",temp);
			pObject ->SetOrientation(temp);
			
			INI->GetValueTrFloat(ObjectName,"position_offset",temp);
			pObject ->SetPositionOffset(temp);
			
			INI->GetValueTrFloat(ObjectName,"orientation_offset",temp);
			pObject ->SetOrientationOffset(temp);
			
			INI->GetValueTrFloat(ObjectName,"color",temp);
			temp[3]=INI->GetValueDouble(ObjectName,"color_alpha");
			pObject ->SetColor(temp);
			
			GeometrieName=INI->GetValue(ObjectName,"geometrie_file");
			if (GeometrieName.GetLength()>0) pObject ->SetGeometrie(GeometrieName);

			pObject->SetShowOrigin(INI->GetValueInt(ObjectName,"show_origin")>0);
			pObject->SetShowGeometrie(INI->GetValueInt(ObjectName,"show_geometrie")>0);
		}
		else addobjects=false;
	};
}

void COpenGLScene::InitServos()
{	CDlgStatus* pDlgStatus = CDlgStatus::m_pCDlgStatus;
	if (pDlgStatus) pDlgStatus->SetTextB("searching Servos...");
	
	CServo* pServo;
	CIniFile* INI = CIniFile::m_pCIniFile;
	
	if (INI->IsKey("servo1"))
	{	if (pDlgStatus) pDlgStatus->SetTextB("lade Servos1");
		pServo = new CServo;
	};
	if (INI->IsKey("servo2"))
	{	if (pDlgStatus) pDlgStatus->SetTextB("lade Servos2");
		pServo = new CServo;
	};
	if (INI->IsKey("servo3"))
	{	if (pDlgStatus) pDlgStatus->SetTextB("lade Servos3");
		pServo = new CServo;
	};
	if (INI->IsKey("servo4"))
	{	if (pDlgStatus) pDlgStatus->SetTextB("lade Servos4");
		pServo = new CServo;
	};
	if (INI->IsKey("servo5"))
	{	if (pDlgStatus) pDlgStatus->SetTextB("lade Servos5");
		pServo = new CServo;
	};
	if (INI->IsKey("servo6"))
	{	if (pDlgStatus) pDlgStatus->SetTextB("lade Servos6");
		pServo = new CServo;
	};
}

void COpenGLScene::InitPlattform()
{	CPlattform* pPlattform;
	CIniFile* INI = CIniFile::m_pCIniFile;
	
	CDlgStatus* pDlgStatus = CDlgStatus::m_pCDlgStatus;
	if (pDlgStatus) pDlgStatus->Show();
	if (pDlgStatus) pDlgStatus->SetTextA("InitPlattform() ...");
	if (pDlgStatus) pDlgStatus->SetTextB("");	
	if (INI->IsKey("plattform")) pPlattform = new CPlattform;
}

void COpenGLScene::DrawServoDebug()
{
	CServo* pServo=NULL;
	int i,n;															//hilfsvariable	
	double dRadius;
	float fPos[3];
	float fOrient[3];
	float fLinkPointLokal[3];
	float fLinkPointGlobal[3];
	
	glDisable(GL_LIGHTING);
	for (i=1;i<=6;i++)
	{	pServo=CServo::GetServo(i);
		if (pServo)
		{
			int startwinkel=180-(int)pServo->GetStartAngle();
			int winkel=180-(int)pServo->GetAngle()-(int)pServo->GetStartAngle();
			
			dRadius=pServo->GetArmLength();
			
			pServo->GetPosition(fPos);
			pServo->GetOrientation(fOrient);
			pServo->GetLinkPointLokal(fLinkPointLokal);
			pServo->GetLinkPointGlobal(fLinkPointGlobal);

			glPushMatrix();
				glTranslatef(fPos[0],fPos[1],fPos[2]);
				glRotatef(fOrient[0],0.0,1.0,0.0);
				glRotatef(fOrient[1],-1.0,0.0,0.0);
				glRotatef(fOrient[2],0.0,0.0,1.0);
				glPointSize(2.0);
				glLineWidth(1.0);
				glColor3d(0.0,1.0,0.0);
				glBegin(GL_LINE_LOOP);					//Kreisboogen zeichnen
					glVertex3d(0.0,0.0,0.0);
					for(n=winkel;n<=startwinkel;n+=10) 
						{	glVertex3d((dRadius*cos(n*CServo::piover180)),(dRadius*sin(n*CServo::piover180)),0.0);
						}
					n=startwinkel;	
					glVertex3d((dRadius*cos(n*CServo::piover180)),(dRadius*sin(n*CServo::piover180)),0.0);
					glVertex3d(0.0,0.0,0.0);
				glEnd();

				glLineWidth(3.0);						//HebelArm darstellen
				glColor3d(0.0,1.0,0.0);
				glBegin(GL_LINES);
					glVertex3d(0.0,0.0,0.0);
					glVertex3f(fLinkPointLokal[0],fLinkPointLokal[1],fLinkPointLokal[2]);				
				glEnd();
				glPointSize(4.0);
				glColor3d(0.2,1.0,0.0);					//LinkPoint darstellen
				glBegin(GL_POINTS);
					glVertex3f(fLinkPointLokal[0],fLinkPointLokal[1],fLinkPointLokal[2]);				
				glEnd();
			glPopMatrix();
		};
	};
	if (m_bLight) glEnable(GL_LIGHTING);				
}

void COpenGLScene::DrawPlattformDebug()
{
	CPlattform* pPlattform=NULL;
	
//	int i,n;															//hilfsvariable	
	
	float fPos[3];
	float fOrient[3];
	float fLPlok[6][3];
	float fLPglob[6][3];
	float LP[3];
	float L=1.0f;
	int n;
	glDisable(GL_LIGHTING);
	pPlattform=CPlattform::m_pCPlattform;
	
	if (pPlattform)
	{
		pPlattform->GetPosition(fPos);
		pPlattform->GetOrientation(fOrient);
		
		pPlattform->GetLinkPointLokal(1,fLPlok[0]);
		pPlattform->GetLinkPointLokal(2,fLPlok[1]);
		pPlattform->GetLinkPointLokal(3,fLPlok[2]);
		pPlattform->GetLinkPointLokal(4,fLPlok[3]);
		pPlattform->GetLinkPointLokal(5,fLPlok[4]);
		pPlattform->GetLinkPointLokal(6,fLPlok[5]);
		
		pPlattform->GetLinkPointGlobal(1,fLPglob[0]);
		pPlattform->GetLinkPointGlobal(2,fLPglob[1]);
		pPlattform->GetLinkPointGlobal(3,fLPglob[2]);
		pPlattform->GetLinkPointGlobal(4,fLPglob[3]);
		pPlattform->GetLinkPointGlobal(5,fLPglob[4]);
		pPlattform->GetLinkPointGlobal(6,fLPglob[5]);

/*		glPushMatrix();
			glTranslatef(fPos[0],fPos[1],fPos[2]);
			glRotatef(fOrient[0],0.0,1.0,0.0);
			glRotatef(fOrient[1],-1.0,0.0,0.0);
			glRotatef(fOrient[2],0.0,0.0,1.0);
			
			glPointSize(5.0);
			glLineWidth(1.0);
			glColor3d(0.0,1.0,0.0);
			glBegin(GL_POINTS);
				glVertex3d(fLPlok[0][0],fLPlok[0][1],fLPlok[0][2]);
				glVertex3d(fLPlok[1][0],fLPlok[1][1],fLPlok[1][2]);
				glVertex3d(fLPlok[2][0],fLPlok[2][1],fLPlok[2][2]);
				glVertex3d(fLPlok[3][0],fLPlok[3][1],fLPlok[3][2]);
				glVertex3d(fLPlok[4][0],fLPlok[4][1],fLPlok[4][2]);
				glVertex3d(fLPlok[5][0],fLPlok[5][1],fLPlok[5][2]);
			glEnd();
		glPopMatrix();
*/		glPushMatrix();
			glLineWidth(1);
			glColor3d(0.6,0.6,0.0);
			glBegin(GL_LINES);
				for (n=0;n<=5;n++)
				{	LP[0]=fLPglob[n][0];
					LP[1]=fLPglob[n][1];
					LP[2]=fLPglob[n][2];
					glVertex3f(LP[0]-L,LP[1],LP[2]);
					glVertex3f(LP[0]+L,LP[1],LP[2]);
					glVertex3f(LP[0],LP[1]-L,LP[2]);
					glVertex3f(LP[0],LP[1]+L,LP[2]);
					glVertex3f(LP[0],LP[1],LP[2]-L);
					glVertex3f(LP[0],LP[1],LP[2]+L);
				};
			glEnd();
			glLineWidth(3);
			glColor3d(0.0,1.0,0.0);
			glBegin(GL_LINE_LOOP);
				for (n=0;n<=5;n++)
					glVertex3f(fLPglob[n][0],fLPglob[n][1],fLPglob[n][2]);
			glEnd();
		glPopMatrix();
	};
	if (m_bLight) glEnable(GL_LIGHTING);	
}

void COpenGLScene::DrawLinkDebug()
{
	
//	CPlattform* pPlattform=NULL;
	CServo* pServo=NULL;

	float fPpos[3];
	float fCpos[3];
	float fSpos[3];
	
	for (int n=1;n<=6;n++)
	{
		pServo=CServo::GetServo(n);
		if (pServo != NULL)
		{	pServo->getPlattformPointGlobal(fPpos);
			pServo->GetPosition(fSpos);
			pServo->GetLinkPointGlobal(fCpos);
		}
		else//added by S.Wei 03.09.2007
		{
			return;
		}
		glDisable(GL_LIGHTING);

		glPushMatrix();
			glLineWidth(3.0);
			glPointSize(5.0);
			if ((pServo != NULL) && (pServo->IsLinked())) //changed by S.Wei 03.09.2007
				glColor3d(0.0,1.0,0.0);
			else 
				glColor3d(1.0,0.0,0.0);
			glBegin(GL_POINTS);
				glVertex3d(fPpos[0],fPpos[1],fPpos[2]);
				glVertex3d(fSpos[0],fSpos[1],fSpos[2]);
				glVertex3d(fCpos[0],fCpos[1],fCpos[2]);
			glEnd();
			glBegin(GL_LINES);
				glVertex3d(fPpos[0],fPpos[1],fPpos[2]);
				glVertex3d(fCpos[0],fCpos[1],fCpos[2]);
				glVertex3d(fCpos[0],fCpos[1],fCpos[2]);
				glVertex3d(fSpos[0],fSpos[1],fSpos[2]);
			glEnd();
		glPopMatrix();
	};
	if (m_bLight) glEnable(GL_LIGHTING);	
}
