﻿#pragma once

class CServo  
{
public:
 void	SetLinked(bool doit)	{m_bLinked=doit;};
 bool	IsLinked() const	{return m_bLinked;};
	
 void	getPlattformPointLokal(float pos[3]) const	{Copy3F(pos,m_fPlattformPointLokal);};
 void	setPlattformPointLokal(const float pos[3])	{Copy3F(m_fPlattformPointLokal,pos);};
 void	getPlattformPointGlobal(float pos[3]) const	{Copy3F(pos,m_fPlattformPointGlobal);};
 void	setPlattformPointGlobal(const float pos[3])	{Copy3F(m_fPlattformPointGlobal,pos);};
	
	bool	CalcConnection();

 int	GetUsedAngle()			const	{return m_nUsedAngle;};
 void	SetUsedAngle(int nAngle)		{m_nUsedAngle=nAngle;};
 float	GetArmLength()			const	{return m_fArmLength;};			// liefert die Länge des Hebelarms
 void	SetArmLength(float fLength)		{m_fArmLength=fLength;};		// setzt die Länge des Hebelarms
 float	GetLinkLength()			const	{return m_fLinkLength;};		// liefert die Länge der Verbindungsstrebe
 void	SetLinkLength(float fLength)		{m_fLinkLength=fLength;};		// setzt die Länge der Verbindungsstrebe
 float	GetStartAngle()			const	{return m_fStartAngle;};		// liefert den virtuellen "NullWinkel" für das Zeichnen
 void	SetStartAngle(float fAngle)		{m_fStartAngle=fAngle;};		// setzt den virtuellen "NullWinkel" für das Zeichnen
 float	GetAngleCorrectionFactor()	const	{return m_fAngleCorrectionFactor;};	// liefert den Korrekturfaktor für Servowinkel
 void	SetAngleCorrectionFactor(float fFactor)	{m_fAngleCorrectionFactor=fFactor;};	// setzt den Korrekturfaktor für Servowinkel

 static	CServo* CServo::GetServo(int nServoNr);						//liefert Pointer auf nummerierte Instanz
 static	CServo* CServo::GetServo(CString strServoName);					//liefert Pointer auf benannte Instanz

 float	GetAngle()			const	{return m_fAngle;};			// liefert den ServoWinkel
 void	SetAngle(float fAngle);								// setzt den ServoWinkel und berechnet die 3D-Grafik
 void	SetOrientation(const float v[3])	{Copy3F(m_fOrientation,v);Drehmatrix(v,m_A);UpdateServoObjects();};// Einstellen der Servoausrichtung (winkel)
// void _stdcall SetOrientation(float a, float b, float c);					// Einstellen der Servoausrichtung (winkel)
 void	SetPosition(const float v[3])		{Copy3F(m_fPosition,v);UpdateServoObjects();};	// Einstellen der Servoposition (abstände)
 void _stdcall SetPosition(float x, float y, float z);						// Einstellen der Servoposition (abstände)
 void	GetOrientation(float ori[3])	const	{Copy3F(ori,m_fOrientation);};		// liefert die Servoausrichtung (winkel)
 void	GetPosition(float pos[3])	const	{Copy3F(pos,m_fPosition);};		// liefert die Servoposition (abstände)
 void	GetLinkPointLokal(float pos[3])	const	{Copy3F(pos,m_fLinkPointLokal);};	// liefert den Servoabtriebspunkt im Servokoordinatensystem
 void	GetLinkPointGlobal(float pos[3])const	{Copy3F(pos,m_fLinkPointGlobal);};	// liefert den Servoabtriebspunkt im Szenenkoordinatensystem
	
 CString GetName()			const	{return m_strName;};		// liefert die Servobezeichnung ("servo1..servo6")
 int	GetServoNumber()		const	{return m_nServoNumber;};	// liefert die Servonummer ("1..6")
	
 CServo();							// Constructor
 ~CServo();							// Destructor

 static	CList<CServo*,CServo*> m_pServoList;	// statische Liste für dynamische Instanzierung

 void	MoveRealServo() const;			//seit 21.12.2007 nur noch zum Servo testen in Verwendung; Aufgaben werden durch die Methoden GetMoveString() und MoveAllServos() erledigt
 CString GetMoveString() const;			//gibt die 7stellige Befehlszeichenkette zurück
 static bool MoveAllServos();

 static void Copy3F(float*d,const float*s)	{CopyMemory(d,s,sizeof(float)*3);};
 static const float piover180;			// π/180;
 static void Drehmatrix(const float orient[3], float A[3][3]);
 static void MatrixTrafo(const float A[3][3],const float x[3], const float b[3], float y[3]);
private:
 bool m_bLinked;
 float m_fPlattformPointGlobal[3];
 float m_fPlattformPointLokal[3];
 static bool GetCircleIntersection(float dRA, const float dPB[2], float dRB, float dP1[3], float dP2[3]);

 static void TransformG2L(const float y[3], const float dZpos_g[3], const float dZorient[3], float x[3]);

 int	m_nUsedAngle;

 void	UpdateLinkPoint();
 float	m_fLinkPointGlobal[3];			// Servoantriebspunkt in Szenenkoordinaten
 float	m_fLinkPointLokal[3];			// Servoantriebspunkt in Servokoordinaten
 void	InitServoObjects();			// initialisiert Objekte (pos/orient/geometrie/....)
 void	DestroyServoObjects();			// macht den Initialisierungszustand rückgängig (seit 20.12.2007)
 void	UpdateServoObjects();			// aktualisert Position und Orientierung der Anzeige-Objekte

 float	m_fArmLength;				// Hebelarm
 float	m_fLinkLength;				// Strebenlänge der Verbindungsstreben
 float	m_fAngleCorrectionFactor;		// Korrekturfaktor für Winkellage -> 90°=...Incremente
 float	m_fAngleMinIncrements;
 float	m_fAngleMaxIncrements;
 float	m_fStartAngle;				// Nullwinkel fürs Zeichnen
 float	m_fAngle;				// eingestellter Servowinkel
 float	m_fAngleCalc;				// berechneter Servowinkel
 float	m_fOrientation[3];			// Ausrichtung (winkel)
 float	m_fPosition[3];				// Position (abstände)
 float	m_A[3][3];				// Drehmatrix
 int	m_nServoNumber;				// ServoNummer ("1...6")
 CString m_strName;				// Servobezeichnung ("servo1...servo5")
};
